#ifndef VIDEOFORMATS_H_
#define VIDEOFORMATS_H_

#include <string>
#include <gd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_LIBTHEORAENC
#include <theora/codec.h>
#endif
#include "definition.h"

enum BGRFormat_t {
  rgb_unknown,
  rgba,
  bgra,
  bgr24,
  rgb24,
  bgr16
};

class BasePlane {
  //protected:
  public: /* public to be sure that nothing is broken */
    uint32 width;
    uint32 height;
    uint32 stride;

  public:
    BasePlane()
: width(0), height(0), stride(0) {}

    BasePlane(uint32 _width, uint32 _height, uint32 _stride = 0)
    : width(_width), height(_height), stride(_stride?_stride:_width)
    { }

    virtual ~BasePlane(){};

    virtual void cleanup() = 0;
};

class RGBPlane : public BasePlane {

  public:

    enum SuffixType {
      suffix_unknown,
      suffix_jpg,
      suffix_png,
      suffix_gif
    };

    uint8*      plane;
    BGRFormat_t format;

    void convertToInternal(gdImagePtr im);

    SuffixType identifySuffix(const std::string& filename);

  public:

  RGBPlane()
   : plane(0), format(rgb_unknown) {}
  RGBPlane(uint32 width, uint32 height);

  virtual ~RGBPlane(){};

  bool load(const std::string& filename, uint32 width=0, uint32 height=0, bool useBiggest = true);
  bool save(const std::string& filename, uint32 width=0, uint32 height=0);

  uint8 linearInterpolation(float x, float y, uint32 add);

  RGBPlane resize(float scaleFactor);
  RGBPlane reframe(uint32 newWidth, uint32 newHeight);
  RGBPlane subframe(uint32 newWidth, uint32 newHeight, float offsetWidth, float offsetHeight, float resizeFactor);

#ifdef HAVE_LIBTHEORAENC
  void exportYCrCb_theora(th_ycbcr_buffer& buffer) const;
  void importYCrCb_theora(const th_ycbcr_buffer& buffer);
#endif

  //void setBackground(uint8 red, uint8 green, uint8 blue, uint8 alpha);

  RGBPlane crossfade(RGBPlane& plane, float velocity);

  RGBPlane& operator=(const RGBPlane& plane);

  virtual void cleanup();
};

enum YCbCrFormat_t {
  ycbcr_unknown,
  ycbcr420,
  ycbcr422,
  ycbcr444
};

struct PicPlane {
  uint32 width;
  uint32 height;
  uint32 stride;
  uint8* plane;

  PicPlane()
   : width(0), height(0), stride(0), plane(0) {};
};

class YCbCrPlane : public BasePlane {

protected:
  YCbCrFormat_t format;
  PicPlane yPlane;
  PicPlane cbPlane;
  PicPlane crPlane;

  uint8 linearInterpolation(float x, float y, PicPlane& localPlane);

public:
  YCbCrPlane()
   : BasePlane(0,0), format(ycbcr_unknown) {}

  YCbCrPlane(uint32 _width, uint32 _height)
   : BasePlane(_width, _height), format(ycbcr420)
   {
	  yPlane.plane = new uint8[width*height];
	  cbPlane.plane = new uint8[(width*height+3)/4];
	  crPlane.plane = new uint8[(width*height+3)/4];
   }

#ifdef HAVE_LIBTHEORAENC
  void exportYCbCr_theora(th_ycbcr_buffer& buffer) const;
  void importYCbCr_theora(const th_ycbcr_buffer& buffer);
#endif

  YCbCrPlane resize(float scaleFactor);
  YCbCrPlane reframe(uint32 newWidth, uint32 newHeight);

  virtual void cleanup();

};


#endif /*VIDEOFORMATS_H_*/
